/*
*/

#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "SunAngle.h"
#include "CompassForm.h"
#include "MathLib.h"
#include "DateTime.h"
#include "TimeMgr.h"
#include "SunAngle_defs.h"
#include "Utils.h"
#include "pdouble.h"

static int sunRise;
static int sunSet;

static void normalize (double *x);
static void drawPie (FormPtr form, int sunRise, int sunSet, int nowBearing);

/******************************************************************************/
static
void CompassFormInit(FormPtr form)
{
  // warning-- don't do any drawing in this routine.
  // Also, don't call FrmSetFocus from here (it must be called *after*
  // FrmDrawForm)
}

/******************************************************************************/
static
void CompassFormDeinit(FormPtr form)
{
}


/******************************************************************************/
void drawCircle (Int16 color, Int16 centerX, Int16 centerY, Int16 radius, Boolean fill, int nowBearing)
{
  double x,y;
  Int16 xInt, yInt, yIntn;
  Int16 xLast, yLast, yLastn;
  
  Int16 tickX1, tickY1;
  Int16 tickX2, tickY2;

  //WinSetForeColor(color | (color << 4));
  WinSetForeColor(GRAY1);

  y = 0;
  x = radius;
  xLast = double2Int16(x);
  yLast = double2Int16(y);
  yLastn = double2Int16(-y);

  for (y=0;y<=radius;y++)
  {
    x = radius*tcos(tasin(y/radius));
    xInt = double2Int16(x);
    yInt = double2Int16(y);
    yIntn = double2Int16(-y);
    if (fill) 
    {
      WinDrawLine (-xInt+centerX, yIntn+centerY, xInt+centerX, yIntn+centerY);
      WinDrawLine (-xInt+centerX, yInt+centerY, xInt+centerX, yInt+centerY);
    } else
    { 
      WinDrawLine (xLast+centerX, yLastn+centerY, xInt+centerX, yIntn+centerY);
      WinDrawLine (-xLast+centerX, yLastn+centerY, -xInt+centerX, yIntn+centerY);
      WinDrawLine (xLast+centerX, yLast+centerY, xInt+centerX, yInt+centerY);
      WinDrawLine (-xLast+centerX, yLast+centerY, -xInt+centerX, yInt+centerY);
    }
    xLast = xInt;
    yLast = yInt;
    yLastn = yIntn;
  }

  for (y=-nowBearing;y<360-nowBearing;y+=90) {
    tickX1 = double2Int16(radius*tcos(y*DEGRAD));
    tickY1 = double2Int16(radius*tsin(y*DEGRAD));
    tickX2 = double2Int16((radius-4)*tcos(y*DEGRAD));
    tickY2 = double2Int16((radius-4)*tsin(y*DEGRAD));
    WinDrawLine (tickX1+centerX, tickY1+centerY, tickX2+centerX, tickY2+centerY);
  }
  for (y=-nowBearing;y<360-nowBearing;y+=10) {
    tickX1 = double2Int16((radius-4)*tcos(y*DEGRAD));
    tickY1 = double2Int16((radius-4)*tsin(y*DEGRAD));
    WinDrawPixel (tickX1+centerX, tickY1+centerY);
  }
}

/******************************************************************************/
void drawArrow (double angle, Int16 centerX, Int16 centerY, Int16 length)
{
  Int16 xEndArrow, yEndArrow;
  Int16 xInt, yInt;
  
  WinSetForeColor(15);
//  if (angle > 2*PI) angle -= 2*PI;
//  if (angle < -2*PI) angle += 2*PI;
  xEndArrow = double2Int16(length*cos(angle) + centerX);
  yEndArrow = double2Int16(-length*sin(angle) + centerY);

  WinDrawLine (centerX, centerY, xEndArrow, yEndArrow);

  xInt = double2Int16(10*cos(angle-PI-0.5) + xEndArrow);
  yInt = double2Int16(-10*sin(angle-PI-0.5) + yEndArrow);
  WinDrawLine (xInt, yInt, xEndArrow, yEndArrow);
  xInt = double2Int16(10*cos(angle-PI+0.5) + xEndArrow);
  yInt = double2Int16(-10*sin(angle-PI+0.5) + yEndArrow);
  WinDrawLine (xInt, yInt, xEndArrow, yEndArrow);

}

/******************************************************************************/
void drawN (double angle, Int16 centerX, Int16 centerY, Int16 length)
{
  Int16 xEndArrow, yEndArrow;

  xEndArrow = double2Int16((length+8)*cos(angle) + centerX - 2.5);
  yEndArrow = double2Int16(-(length+8)*sin(angle) + centerY - 3);
  

  if (xEndArrow < centerX) {
  }

  WinDrawChar ('N', xEndArrow, yEndArrow);

}

/******************************************************************************/
void drawPieSlice (Int16 color,
                   double startAngle, 
                   double endAngle, 
                   Int16 centerX, 
                   Int16 centerY, 
                   Int16 length, 
                   Boolean lowerHemi)
{
  double tempAngle;
  double startX, startY, startSlope;
  double endX, endY, endSlope;
  double y;
  double lineStartX;
  double lineEndX;
  double theta;
  double sliceSize;
  
  WinSetForeColor(color);

//  if (startAngle > endAngle) // switch 'em
//  {
//    tempAngle = endAngle;
//    endAngle = startAngle;
//    startAngle = tempAngle;
//  }
  if (startAngle < -PI/2) 
  {
    startAngle = 2*PI + startAngle;
    tempAngle = endAngle;
    endAngle = startAngle;
    startAngle = tempAngle;
  }
  startX = length*cos(startAngle);
  startY = length*sin(startAngle);
  startSlope = startY/startX;

  endX = length*cos(endAngle);
  endY = length*sin(endAngle);
  endSlope = endY/endX;

  // If the slice straddles the top/bottom hemispheres,
  // split it into two draws.
  // Have to truncate because sin(PI) != 0 since PI isn't
  // enough digits.
 
  if ( (double2Int16(startY) < 0) && (double2Int16(endY) > 0)) 
  {
    drawPieSlice (color, 0, endAngle, centerX, centerY, length, false);
    endAngle = 0;
    endX = length*cos(endAngle);
    endY = length*sin(endAngle);
  } else if ( (double2Int16(startY) > 0) && (double2Int16(endY) < 0) )
  {
    drawPieSlice (color, PI, endAngle, centerX, centerY, length, false);
    endAngle = PI;
    endX = length*cos(endAngle);
    endY = length*sin(endAngle);
  }

  //if ( (startAngle - endAngle) > PI)
  sliceSize = endAngle - startAngle;
  normalize(&sliceSize); 
  if ( sliceSize > PI) // its obtuse, special case
  { 
    if (endY<0 && startY<0)  // there's a hemisphere that needs to be filled
    {
      drawPieSlice(color, 0, PI, centerX, centerY, length, false);
      drawPieSlice(color, 0, endAngle, centerX, centerY, length, false);
      endAngle = PI; //startAngle;
      //startAngle = PI;
      startX = length*cos(startAngle);
      startY = length*sin(startAngle);
      endX = length*cos(endAngle);
      endY = length*sin(endAngle);
    }
    if (endY>0 && startY>0)  // there's a hemisphere that needs to be filled
    {
      drawPieSlice(color, 0, PI, centerX, centerY, length, true);
  //    drawPieSlice(color, endAngle, 0, centerX, centerY, length, false);
      endAngle = PI;
      //startAngle = PI;
      startX = length*cos(startAngle);
      startY = length*sin(startAngle);
      endX = length*cos(endAngle);
      endY = length*sin(endAngle);
    }
  }
//  WinDrawLine (centerX, centerY, startX+centerX, -startY+centerY);
//  WinDrawLine (centerX, centerY, endX+centerX, -endY+centerY);

  //for (y=-length; y<length; y++) 
  // We're always given either an upper-hemisphere slice,
  // or a lower... two loops to handle the different cases
  if ((startY >= 0) && (endY >= 0) && !lowerHemi)   // upper hemi
  {
    for (y=0; y<=length; y++) 
    {
      lineStartX = 0;
      lineEndX = 0;
      if ( ((endAngle > PI/2) && (startAngle < PI/2) )  // still drawing
           || ( (y<startY) || (y<endY) ) ) 
      {
        // lineStartX
        if ((y<=endY) && (endSlope))
        { 
          lineStartX = y/endSlope;
        } else 
        {
          theta = asin(y/length);
          lineStartX = -length*tcos(theta); 
        }
        // lineEndX
        if ((y<=startY) && (startSlope))
        {
          lineEndX = y/startSlope;
        } else
        {
          theta = asin(y/length);
          lineEndX = length*tcos(theta);
        }
      WinDrawLine (lineStartX+centerX + 0.5, -y+0.5+centerY, 
                    lineEndX+centerX + 0.5, -y+0.5+centerY );
      }
    }
  } else  // lower hemi
  {
    for (y=-length; y<=0; y++) 
    {
      lineStartX = 0;
      lineEndX = 0;
      if ( ((endAngle > 3*PI/2) && (startAngle < 3*PI/2) ) 
           || ( (y>startY) || (y>endY) ) ) 
      {
        // lineStartX
        if ((y>=startY) && (startSlope))
        { 
          lineStartX = y/startSlope;
        } else 
        {
          theta = asin(y/length);
          lineStartX = -length*tcos(theta); 
        }
        // lineEndX
        if ((y>=endY) && (endSlope))
        {
          lineEndX = y/endSlope;
        } else
        {
          theta = asin(y/length);
          lineEndX = length*tcos(theta);
        }
      WinDrawLine (lineStartX+centerX + 0.5, -y+0.5+centerY, 
                    lineEndX+centerX + 0.5, -y+0.5+centerY);
      }
    }
  }
}

// Strangely, if the application starts with now=true,
// there's a nilEvent early on, and this routine causes
// a "Form.c, Line:1728, Object not in form". SysFatalAlert
// even though the updateTime function is OK.
// The current form seems to be correct, the screen is mostly drawn...
// Bug in POSE, PalmOS?

/******************************************************************************/
static void updateDate (FormPtr form) 
{
  Char *label;

  label = (Char *)FrmGetLabel(form, DateLabel);

  DateToAscii (sunangleInput->dateTime.month,
   sunangleInput->dateTime.day,
   sunangleInput->dateTime.year,
   dfMDYWithSlashes,
   label);
  StrCat(label, "  "); // erase anything trailing from previous value

} 

/******************************************************************************/
static void updateTime (FormPtr form)
{
  Char *label;

  label = (Char *)FrmGetLabel(form, TimeLabel);

  TimeToAscii (sunangleInput->dateTime.hour,
   sunangleInput->dateTime.minute,
   tfColonAMPM,
   label);
  StrCat(label, "  "); // erase anything trailing from previous value

} 

/******************************************************************************/
static Boolean HandleNow(int nowBearing)
{
  DateTimeType oldDateTime;
  FormPtr form;
  Boolean update;
  form = FrmGetActiveForm();
  update = false;

  if (sunangleInput->now)
  { 
    oldDateTime = sunangleInput->dateTime;
    TimSecondsToDateTime (TimGetSeconds(), &sunangleInput->dateTime);
    if (oldDateTime.month != sunangleInput->dateTime.month
        || oldDateTime.day != sunangleInput->dateTime.day
        || oldDateTime.year != sunangleInput->dateTime.year)
    {
  //      updateDate(form);
        update = true;
    }
    if (oldDateTime.minute != sunangleInput->dateTime.minute
        || oldDateTime.hour != sunangleInput->dateTime.hour)
    {
        updateTime(form);
        update = true;
    }
    if (update) 
    {
//      WinSetDrawWindow(WinGetDisplayWindow());
//      WinEraseWindow;
      FrmEraseForm(form);
      FrmDrawForm(form);
      drawPie (form, sunRise, sunSet, nowBearing);
    }
  }
  // we don't return true because we don't pretend to handle the
  // event.  this is because Astro.c doesn't... i think it might
  // be so everyone else sees the nil event.

  return false;

}

/******************************************************************************/
static void normalize (double *x)
{
  while (*x > 2*PI)
    *x -= 2*PI;
  while (*x < -2*PI)
    *x += 2*PI;
  while (*x < 0)
    *x = 2*PI + *x;
}
/******************************************************************************/
static void drawSunRiseSet (Int16 color, int sunRise, int sunSet, int nowBearing)
{
  double sunRiseAngle;
  double sunSetAngle;
  Int16 centerX, centerY;
  Int16 extentX, extentY;
  Int16 radius;

  WinGetWindowExtent (&extentX, &extentY);
  centerX = extentX/2;
  centerY = extentY/2 + 10;
  radius = extentX/2 - 24;

  sunRiseAngle = (-sunRise + nowBearing + 90)*DEGRAD;
  normalize(&sunRiseAngle);
  sunSetAngle = (-sunSet + nowBearing + 90)*DEGRAD;
  normalize(&sunSetAngle);

  drawPieSlice (color, sunSetAngle, sunRiseAngle, centerX, centerY, radius, false);
/*
  // i think a N/S hemisphere difference is needed here
  if ((nowBearing >= 90) && (nowBearing <= 270)) 
  { 
//    DisplayStringsInAlert("daytime", "");
    drawPieSlice (color, sunSetAngle-2*PI, sunRiseAngle, centerX, centerY, radius);
  }
  else 
    drawPieSlice (color, sunRiseAngle, sunSetAngle, centerX, centerY, radius);
*/
}

/******************************************************************************/
static void drawPie (FormPtr form, int sunRise, int sunSet, int nowBearing)
{
  Int16 centerX, centerY;
  Int16 radius;
  Int16 extentX, extentY;
  Int16 i;
  sunangleOutputType now;
  sunangleOutputType winterSunSetOutput;
  sunangleOutputType winterSunRiseOutput;
  sunangleInputType winterSunSetInput;
  sunangleInputType winterSunRiseInput;
//  UInt16 winterSunRise;
//  UInt16 winterSunSet;
  sunangleOutputType summerSunSetOutput;
  sunangleOutputType summerSunRiseOutput;
  sunangleInputType summerSunSetInput;
  sunangleInputType summerSunRiseInput;
//  UInt16 summerSunRise;
//  UInt16 summerSunSet;
//  double sunRiseAngle;
//  double sunSetAngle;

  WinSetDrawWindow(WinGetDisplayWindow());
  WinGetWindowExtent (&extentX, &extentY);
  centerX = extentX/2;
  centerY = extentY/2 + 10;
  radius = extentX/2 - 24;

//  WinEraseWindow;
  FrmEraseForm(form);
  FrmDrawForm(form);
//  drawCircle (0, centerX, centerY, radius+2, true, nowBearing);

  CalcSunangle(sunangleInput, &now, true); // why can't we use input parameter?
  
  summerSunRiseInput = *sunangleInput;
  summerSunRiseInput.dateTime.day = 21;  // actually, this varies
  if (sunangleInput->latitude > 0) {  // northern hemisphere
    summerSunRiseInput.dateTime.month = 6;
  } else {
    summerSunRiseInput.dateTime.month = 12;
  }
  CalcSunangle(&summerSunRiseInput, &summerSunRiseOutput, true);
  summerSunRiseInput.dateTime.hour = summerSunRiseOutput.sunRise.hours;
  summerSunRiseInput.dateTime.minute = summerSunRiseOutput.sunRise.minutes;
  summerSunRiseInput.dateTime.second = 0;
  CalcSunangle(&summerSunRiseInput, &summerSunRiseOutput, true);

  summerSunSetInput = summerSunRiseInput;
  summerSunSetInput.dateTime.hour = summerSunRiseOutput.sunSet.hours;
  summerSunSetInput.dateTime.minute = summerSunRiseOutput.sunSet.minutes;
  CalcSunangle(&summerSunSetInput, &summerSunSetOutput, true);

  winterSunRiseInput = *sunangleInput;
  winterSunRiseInput.dateTime.day = 21;  // actually, this varies
  if (sunangleInput->latitude > 0) {  // northern hemisphere
    winterSunRiseInput.dateTime.month = 12;
  } else {
    winterSunRiseInput.dateTime.month = 6;
  }
  CalcSunangle(&winterSunRiseInput, &winterSunRiseOutput, true);
  winterSunRiseInput.dateTime.hour = winterSunRiseOutput.sunRise.hours;
  winterSunRiseInput.dateTime.minute = winterSunRiseOutput.sunRise.minutes;
  winterSunRiseInput.dateTime.second = 0;
  CalcSunangle(&winterSunRiseInput, &winterSunRiseOutput, true);

  winterSunSetInput = winterSunRiseInput;
  winterSunSetInput.dateTime.hour = winterSunRiseOutput.sunSet.hours;
  winterSunSetInput.dateTime.minute = winterSunRiseOutput.sunSet.minutes;
  CalcSunangle(&winterSunSetInput, &winterSunSetOutput, true);
/*
  drawPieSlice (4, PI/8, 5*PI/8, centerX, centerY, radius);
  drawPieSlice (10, PI/4, 3*PI/8, centerX, centerY, radius); 
  drawPieSlice (15, 3*PI/4, 7*PI/8, centerX, centerY, radius); 
  drawPieSlice (15, 14*PI/8, 15*PI/8, centerX, centerY, radius); 
  drawPieSlice (15, 10*PI/8, 11*PI/8, centerX, centerY, radius); 
  drawPieSlice (4, -PI/8, +PI/8, centerX, centerY, radius);
  drawPieSlice (4, 7*PI/8, 9*PI/8, centerX, centerY, radius);
*/
  if (CtlGetValue(GetObjectPtr(form, SummerCheckBox)))
    drawSunRiseSet(GRAY1, summerSunRiseOutput.bearing, summerSunSetOutput.bearing, now.bearing);
  if (CtlGetValue(GetObjectPtr(form, TodayCheckBox)))
    drawSunRiseSet(GRAY2, sunRise, sunSet, now.bearing);
  if (CtlGetValue(GetObjectPtr(form, WinterCheckBox)))
    drawSunRiseSet(GRAY3, winterSunRiseOutput.bearing, winterSunSetOutput.bearing, now.bearing);
  //drawSunRiseSet(12, 90, 120, now.bearing);

//  WinSetForeColor(0xFF);
//  for (i=0;i<5;i++) {
//    WinDrawLine (centerX-i, 16+i, centerX+i, 16+i);
//  }
//    printDouble(now.bearing, temp);
//    DisplayStringsInAlert("bearing:", temp);
  drawArrow ((PI/2)+(now.bearing*DEGRAD), centerX, centerY, radius-1);
  //drawN (2*PI-(PI-now.azimuth*DEGRAD), centerX, centerY, radius-1);
  drawN ((PI/2)+(now.bearing*DEGRAD), centerX, centerY, radius-1);
  drawCircle (15, centerX, centerY, radius, false, nowBearing);
}

/******************************************************************************/
static Boolean  penWithinObj (FormPtr form, EventPtr event, UInt16 objID)
{
  RectangleType rect;
  FrmGetObjectBounds(form, FrmGetObjectIndex(form, objID), &rect);
  if ((event->screenX >= rect.topLeft.x) &&
       (event->screenX <= rect.topLeft.x+rect.extent.x) &&
       (event->screenY >= rect.topLeft.y) &&
       (event->screenY <= rect.topLeft.y+rect.extent.y))
    return true;
  else
    return false;
}


/******************************************************************************/
static void flipCheck (FormPtr form, UInt16 objID)
{
  Int16 checkValue;
  checkValue = CtlGetValue(GetObjectPtr(form, objID));
  checkValue = checkValue ^ 1;
  CtlSetValue(GetObjectPtr(form, objID), checkValue);
}
/******************************************************************************/

Boolean CompassFormHandleEvent(EventPtr event)
{
  Boolean handled = false;
  FormPtr form;
  sunangleInputType tempSunAngleInput;
  sunangleOutputType tempSunAngleOutput;
  sunangleOutputType now;



  form = FrmGetActiveForm();
  switch (event->eType)
  {
  case nilEvent:
    CalcSunangle(sunangleInput, &now, true);
    handled = HandleNow(now.bearing);
  break;
  case frmOpenEvent:
    CompassFormInit(form);
    updateDate(form);
    updateTime(form);
    FrmDrawForm(form);
    CalcSunangle(sunangleInput, &tempSunAngleOutput, true);
    // can't we just copy struct?
    tempSunAngleInput.latitude = sunangleInput->latitude; 
    tempSunAngleInput.longitude = sunangleInput->longitude; 
    tempSunAngleInput.timeZone = sunangleInput->timeZone; 
    tempSunAngleInput.dst = sunangleInput->dst; 
    tempSunAngleInput.now = sunangleInput->now; 
    tempSunAngleInput.dateTime.second = 0;
    tempSunAngleInput.dateTime.minute = tempSunAngleOutput.sunRise.minutes;
    tempSunAngleInput.dateTime.hour = tempSunAngleOutput.sunRise.hours;
    tempSunAngleInput.dateTime.day = sunangleInput->dateTime.day;
    tempSunAngleInput.dateTime.month = sunangleInput->dateTime.month;
    tempSunAngleInput.dateTime.year = sunangleInput->dateTime.year;
    CalcSunangle(&tempSunAngleInput, &tempSunAngleOutput, true);
    sunRise = tempSunAngleOutput.bearing;
    tempSunAngleInput.dateTime.minute = tempSunAngleOutput.sunSet.minutes;
    tempSunAngleInput.dateTime.hour = tempSunAngleOutput.sunSet.hours;
    CalcSunangle(&tempSunAngleInput, &tempSunAngleOutput, true);
    sunSet = tempSunAngleOutput.bearing;
//    StrPrintF(temp, "Sunrise: %d  Sunset: %d", sunRise, sunSet);
//    DisplayStringsInAlert("", temp);
    CalcSunangle(sunangleInput, &now, true);

    if (sunangleInput->TodayCheck)
      CtlSetValue(GetObjectPtr(form, TodayCheckBox), 1);
    if (sunangleInput->SummerCheck)
      CtlSetValue(GetObjectPtr(form, SummerCheckBox), 1);
    if (sunangleInput->WinterCheck)
      CtlSetValue(GetObjectPtr(form, WinterCheckBox), 1);

    drawPie(form, sunRise, sunSet, now.bearing);
    handled = true;
    break;
  case penDownEvent:
    if (penWithinObj(form, event, WinterLabel))
    {
      flipCheck (form, WinterCheckBox);
      sunangleInput->WinterCheck = CtlGetValue(GetObjectPtr(form, WinterCheckBox));
      drawPie (form, sunRise, sunSet, now.bearing);
      handled = true;
    }
    if (penWithinObj(form, event, SummerLabel))
    {
      flipCheck (form, SummerCheckBox);
      sunangleInput->SummerCheck = CtlGetValue(GetObjectPtr(form, SummerCheckBox));
      drawPie (form, sunRise, sunSet, now.bearing);
      handled = true;
    }
    if (penWithinObj(form, event, TodayLabel))
    {
      flipCheck (form, TodayCheckBox);
      sunangleInput->TodayCheck = CtlGetValue(GetObjectPtr(form, TodayCheckBox));
      drawPie (form, sunRise, sunSet, now.bearing);
      handled = true;
    }
    break; 

  case ctlSelectEvent:
  switch (event->data.ctlSelect.controlID) {
    case b_Info:
      FrmGotoForm(InfoForm);
      handled = true;
      break;
    case b_Alt:
      FrmGotoForm(AltForm);
      handled = true;
      break;
    case b_Compass:
      handled = true;
      break;
    case b_Listing:
      FrmGotoForm(ListingForm);
      handled = true;
      break;
    case b_Settings:
      FrmGotoForm(SettingsForm);
      handled = true;
      break;
    case WinterCheckBox:
      drawPie (form, sunRise, sunSet, now.bearing);
    break;
    case SummerCheckBox:
      drawPie (form, sunRise, sunSet, now.bearing);
    break;
    case TodayCheckBox:
      drawPie (form, sunRise, sunSet, now.bearing);
    break;
    }
    break;

  case frmCloseEvent:
    CompassFormDeinit(FrmGetActiveForm());
    handled = false;
    break;

  default:
    break;
  }
  return handled;
}
