/*
 Copyright (c) 2000-2001, Neil Rhodes and Julie McKeehan
   neil@pobox.com
 All rights reserved.

From the book "Palm Programming, the Developer's Guide, 2nd edition"
by O'Reilly.

Permission granted to use this file however you see fit.
*/

#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "SunAngle_defs.h"
#include "ListingForm.h"
#include "MathLib.h"
#include "DateTime.h"
#include "TimeMgr.h"
#include "SunAngle.h"
#include "Utils.h"
#include "pdouble.h"

void CalcSunangle (sunangleInputPtr ins, sunangleOutputType* now, Boolean fullCalc);
void printRoundDouble (double x, int digits, Char *str);

static
void ListingFormInit(FormPtr form)
{
  // warning-- don't do any drawing in this routine.
  // Also, don't call FrmSetFocus from here (it must be called *after*
  // FrmDrawForm)
}

static
void ListingFormDeinit(FormPtr form)
{
}


Boolean ListingFormHandleEvent(EventPtr event)
{
  Boolean handled = false;
  FormPtr form;
//  char quantityString[100];
//  char tempString[100];
//  FieldPtr field;
  sunangleOutputType Now;
  Char *label;

  switch (event->eType)
  {
  case frmOpenEvent:
    form = FrmGetActiveForm();
    ListingFormInit(form);
    CalcSunangle(sunangleInput, &Now, true);
//    field = (FieldType *) FrmGetObjectPtr(form, FrmGetObjectIndex(form, CalcData));
//    StrIToA(quantityString, (Int32) Now.declination);
//    StrPrintF (quantityString, "%1d", Now.declination);
//    FlpFToA(Now.declination.fd, quantityString);
//    StrPrintF (quantityString, "%ld", Now.declination.d);
//    FlpBase10Info(Now.declination.fd, &mantissa, &exponent, &sign);
    label = (Char *)FrmGetLabel(form, l_Altitude);
    printRoundDouble(Now.altitude, 2, label);
    label = (Char *)FrmGetLabel(form, l_Azimuth);
    printRoundDouble(Now.azimuth, 2, label);
    label = (Char *)FrmGetLabel(form, l_Declination);
    printRoundDouble(Now.declination, 2, label);
    label = (Char *)FrmGetLabel(form, l_Sunrise);
    TimeToAscii (Now.sunRise.hours, Now.sunRise.minutes, tfColonAMPM, label);
    label = (Char *)FrmGetLabel(form, l_Sunset);
    TimeToAscii (Now.sunSet.hours, Now.sunSet.minutes, tfColonAMPM, label);
    label = (Char *)FrmGetLabel(form, l_Bearing);
    StrPrintF(label, "%d", Now.bearing);
    
//    StrIToA(quantityString, (Int32) mantiss);
//    StrPrintF (quantityString, "mn:%li exp:%i sign:%i", mantissa, exponent, sign);

//    SetFieldTextFromStr (field, quantityString, true);

//    FrmSetFocus(form, FrmGetObjectIndex(form, CalcData));
//    FldSetSelection(field, 0, (UInt16) StrLen(quantityString));
    FrmDrawForm(form);
    // here's where you'd add a call to FrmSetFocus

    handled = true;
    break;


  case ctlSelectEvent:
  switch (event->data.ctlSelect.controlID) {
    case b_Info:
      FrmGotoForm(InfoForm);
      handled = true;
      break;
    case b_Alt:
      FrmGotoForm(AltForm);
      handled = true;
      break;
    case b_Compass:
      FrmGotoForm(CompassForm);
      handled = true;
      break;
    case b_Listing:
      handled = true;
      break;
    case b_Settings:
      FrmGotoForm(SettingsForm);
      handled = true;
      break;
    }

  case frmCloseEvent:
    ListingFormDeinit(FrmGetActiveForm());
    handled = false;
    break;

  default:
    break;
  }
  return handled;
}

void printRoundDouble (double x, int digits, Char *str)
{
  Char tempStr[20];
  Char *tempPtr;
  Char *tempPtr2;

  if (digits > 0)
    x = x + (0.5/(pow(10, digits)));

  printDouble(x, tempStr);

  tempPtr = StrStr(tempStr, ".");

  if (tempPtr)
  {
    tempPtr2 = tempPtr + digits + 1; 
    *tempPtr2 = '\0'; 
  }
  StrCopy(str, tempStr);
}

