/*
*/

#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "SunAngle_defs.h"
#include "InfoForm.h"
#include "AltForm.h"
#include "SettingsForm.h"
#include "ListingForm.h"
#include "CompassForm.h"
#include "Utils.h"
#include "Constants.h"
#include "MathLib.h" 
#include "SunAngle.h"
#include "Grayscale.h"

sunangleInputPtr sunangleInput;
DmOpenRef settingsDBRef;
IndexedColorType GRAY1;
IndexedColorType GRAY2;
IndexedColorType GRAY3;


/******************************************************************************/
static Boolean AppHandleEvent(EventPtr event)
{
  UInt16 formId;
  FormPtr form;

  if (event->eType == frmLoadEvent) {
    // Load the form resource.
    formId = event->data.frmLoad.formID;
    form = FrmInitForm(formId);
    ErrFatalDisplayIf(!form, "Can't initialize form");
    FrmSetActiveForm(form);

    // Set the event handler for the form.  The handler of the currently
    // active form is called by FrmHandleEvent each time is receives an
    // event.
    switch (formId) {
    case ListingForm:
      FrmSetEventHandler(form, ListingFormHandleEvent);
      break;

    case SettingsForm:
      FrmSetEventHandler(form, SettingsFormHandleEvent);
      break;

    case CompassForm:
      FrmSetEventHandler(form, CompassFormHandleEvent);
      break;

    case InfoForm:
      FrmSetEventHandler(form, InfoFormHandleEvent);
      break;

    case AltForm:
      FrmSetEventHandler(form, AltFormHandleEvent);
      break;


    default:
      ErrFatalDisplay("Invalid Form Load Event");
      break;

    }
    return true;
  } else 
    return false;
}


/******************************************************************************/
static 
void AppEventLoop(void)
{
  Err error;
  EventType event;
  EventType newEvent;
  Int32       timeout;
  Int32       ticksPerSec = SysTicksPerSecond();

  do {
    if (sunangleInput->now) {
    // Wait until the next minute starts
      timeout = (60 - (TimGetSeconds() % 60)) * ticksPerSec;
    } else
      timeout = evtWaitForever;
    
    EvtGetEvent(&event, timeout);
    if ( (event.eType == keyDownEvent) 
           && (event.data.keyDown.chr==288) 
           && (event.data.keyDown.modifiers == commandKeyMask)) { // power on = determined w/ DDD
//    DisplayStringsInAlert("Welcome back", "");
    newEvent.eType = nilEvent;
    EvtAddEventToQueue(&newEvent);
  } 
    //SndPlaySystemSound(sndClick);    
    if (! SysHandleEvent(&event))
      if (! MenuHandleEvent(0, &event, &error))
        if (! AppHandleEvent(&event))
          FrmDispatchEvent(&event);

  } while (event.eType != appStopEvent);
}

/******************************************************************************/
Err OpenDB(DmOpenRef *refPtr, UInt16 mode)
{
  Err err = 0;
  DmOpenRef ref = 0;

  // Find the settings database.  If it doesn't exist, create it.
  ref = DmOpenDatabaseByTypeCreator('DATA', 'GBEL', mode);
  if (!ref) {
    err = DmGetLastErr();
    if (err == dmErrCantFind) {
      err = DmCreateDatabase(0, "SunAngleSettings-GBEL", 'GBEL', 'DATA', false);
      if (err)
        return err;
      ref = DmOpenDatabaseByTypeCreator('DATA', 'GBEL', mode);
      DisplayStringsInAlert ("Settings Database Created", "");
      if (!ref)
        err = DmGetLastErr();
    }
  }
//  if (ref) 
//   DisplayStringsInAlert ("Settings Database Opened", "");
  *refPtr = ref;
  return err;
}

/******************************************************************************/
static 
Err AppStart(void)
{
  Err error;
  MemHandle fontHandle;
  MemHandle settingsHandle;
  MemHandle newRecordHandle;
  sunangleInputType *settingsPtr;
  MemPtr settingsMemPtr;
  FontType *fontPtr;
  UInt16 recordIndex;
  RGBColorType *color;

  fontHandle=DmGetResource('NFNT',fnt_Tiny);
  fontPtr=MemHandleLock(fontHandle);
  FntDefineFont(130, fontPtr);

  error = SysLibFind(MathLibName, &MathLibRef);
  if (error) // library not yet loaded, try
    error = SysLibLoad(LibType, MathLibCreator, &MathLibRef);
  if (error) {
    FrmAlert (NoMathLibAlert);
    return error;
  }
  error = MathLibOpen(MathLibRef, MathLibVersion);
  ErrFatalDisplayIf(error, "Error opening MathLib.  Please report this error to gbell@yahoo.com");

  initTrigTables();

  error = OpenDB(&settingsDBRef, dmModeReadWrite);
  ErrFatalDisplayIf(error, "Error opening settings database.  Please report this error to gbell@yahoo.com");
  sunangleInput = MemPtrNew(sizeof(sunangleInputType)); 
  ErrFatalDisplayIf (!sunangleInput, "Out of memory");
//  DmDeleteRecord(settingsDBRef, 0); // clear previous, differently sized record
  settingsHandle = DmQueryRecord(settingsDBRef, 0);
  if (!settingsHandle)
  {
    sunangleInput->latitude = 32;
    sunangleInput->longitude = -117;
    sunangleInput->dst = true;
    sunangleInput->now = true;
    sunangleInput->timeZone = 29;
    sunangleInput->dateTime.hour = 10;
    sunangleInput->dateTime.minute = 0;
    sunangleInput->dateTime.month = 9;
    sunangleInput->dateTime.day = 9;
    sunangleInput->dateTime.year = 2002;
    sunangleInput->TodayCheck = true;
    sunangleInput->SummerCheck = false;
    sunangleInput->WinterCheck = false;
    recordIndex = 0;
    newRecordHandle = DmNewRecord(settingsDBRef, &recordIndex, sizeof(sunangleInputType));
    ErrFatalDisplayIf(!newRecordHandle, "Error getting new record handle");
    settingsMemPtr = (sunangleInputType *)MemHandleLock(newRecordHandle);
    // do we really need to write the record, or is just making a newRecord
    // enough?  try deleting this code
    error = DmWrite(settingsMemPtr, 0, sunangleInput, sizeof(sunangleInputType));
    MemHandleUnlock(newRecordHandle);
    DmReleaseRecord(settingsDBRef, recordIndex, true);
  } else // Settings record exists, read it
  {
    settingsMemPtr = MemHandleLock(settingsHandle);
    settingsPtr = (sunangleInputType *) settingsMemPtr;
    sunangleInput->latitude = settingsPtr->latitude;
    sunangleInput->longitude = settingsPtr->longitude;
    sunangleInput->dst = settingsPtr->dst;
    sunangleInput->now = settingsPtr->now;
    sunangleInput->timeZone = settingsPtr->timeZone;
    sunangleInput->dateTime.hour = settingsPtr->dateTime.hour;
    sunangleInput->dateTime.minute = settingsPtr->dateTime.minute;
    sunangleInput->dateTime.month = settingsPtr->dateTime.month;
    sunangleInput->dateTime.day = settingsPtr->dateTime.day;
    sunangleInput->dateTime.year = settingsPtr->dateTime.year;
    sunangleInput->TodayCheck = settingsPtr->TodayCheck;
    sunangleInput->SummerCheck = settingsPtr->SummerCheck;
    sunangleInput->WinterCheck = settingsPtr->WinterCheck;
    MemHandleUnlock(settingsHandle);
  }
 
  // Set time to now right away to avoid double-update of compass form 
  if (sunangleInput->now) {
    TimSecondsToDateTime (TimGetSeconds(), &sunangleInput->dateTime);
  }

  EnableGraphicsMode(); 
  color = MemPtrNew(sizeof(RGBColorType)); 
  color->r = 0x11; color->g = 0x11; color->b = 0x11;
  GRAY1 = WinRGBToIndex(color);
  color->r = 0x66; color->g = 0x66; color->b = 0x66;
  GRAY2 = WinRGBToIndex(color);
  color->r = 0xBB; color->g = 0xBB; color->b = 0xBB;
  GRAY3 = WinRGBToIndex(color);
  MemPtrFree(color);
  return error; //errNone;
}


/******************************************************************************/
static 
void AppStop(void)
{
  UInt16 usecount;
  Err error;
  MemHandle fontHandle;
  MemHandle settingsHandle;
  MemPtr    settingsMemPtr;
 
  FrmCloseAllForms();
 
  fontHandle = DmGetResource('NFNT',fnt_Tiny);
  MemHandleUnlock(fontHandle);

  // Save settings
  settingsHandle = DmQueryRecord(settingsDBRef, 0);
  ErrFatalDisplayIf(!settingsHandle, "Error getting record handle for saving");
  settingsMemPtr = (sunangleInputType *)MemHandleLock(settingsHandle);
  error = DmWrite(settingsMemPtr, 0, sunangleInput, sizeof(sunangleInputType));
  ErrFatalDisplayIf(error, "Can't close Settings Database");

  // Release record, close database
  MemHandleUnlock(settingsHandle);
  DmReleaseRecord(settingsDBRef, 0, true);
  error = DmCloseDatabase(settingsDBRef);
  ErrFatalDisplayIf(error, "Can't close Settings Database");

  // Free up global, mathlib
  MemPtrFree(sunangleInput);
  error = MathLibClose(MathLibRef, &usecount);
  ErrFatalDisplayIf(error, "Can't close MathLib");
  if (usecount == 0)
      SysLibRemove(MathLibRef); 
}



/******************************************************************************/
UInt32 PilotMain(UInt16 launchCode, MemPtr launchParameters, 
  UInt16 launchFlags)
{
  Err error;

  switch (launchCode) {
  case sysAppLaunchCmdNormalLaunch:
    error = RomVersionCompatible (kOurMinVersion, launchFlags);
    if (error) 
      return error;
    error = AppStart();
    if (error) 
      return error;
      
    FrmGotoForm(CompassForm);
    //FrmGotoForm(AltForm);
    AppEventLoop();
    AppStop();
    break;

  default:
    break;
  }
  
  return errNone;
}


