/*
 Copyright (c) 2000-2001, Neil Rhodes and Julie McKeehan
   neil@pobox.com
 All rights reserved.

From the book "Palm Programming, the Developer's Guide, 2nd edition"
by O'Reilly.

Permission granted to use this file however you see fit.
*/

#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "SunAngle_defs.h"
#include "ListingForm.h"
#include "MathLib.h"
#include "SelDay.h"
#include "DateTime.h"
#include "TimeMgr.h"
#include "StringMgr.h"
#include "SunAngle.h"
#include "ErrorMgr.h"
#include "PalmCompatibility.h"
#include "Utils.h"

//extern  sunangleInputType sunangleInputs;

static void updateDate(FormPtr form, Boolean update);
static void updateTime(FormPtr form, Boolean update);

static
void SettingsFormInit(FormPtr form)
{
  // warning-- don't do any drawing in this routine.
  // Also, don't call FrmSetFocus from here (it must be called *after*
  // FrmDrawForm)
}

static
void SettingsFormDeinit(FormPtr form)
{
}


MemHandle tmpH3;
MemHandle tmpH4;
sunangleInputType newSunAngleInput;

typedef struct {
  char *fullname;
  char *abbrev;
  char *dstabbrev;
  int hours;
  int minutes;
} TimeZone;

TimeZone Zones[] = {
	{ "America/Pacific", 	"PST", "PDT", -8, -8*60},
	{ "America/Central", 	"CST", "CDT", -6, -6*60},
	{ "America/Eastern", 	"EST", "EDT", -5, -5*60},
	{ "US/Alaska", 		"AKST","AKDT",-9, -9*60},
	{ "US/Hawaii-Aleutian",	"HAST","HADT",-10, -10*60},
	{ "America/Mountain", 	"MST", "MDT", -7,  -7*60},
	{ "Canada/Pacific", 	"PST", "PDT", -8,  -8*60},
	{ "America/Havana", 	"CST", "CDT", -5,  -5*60},
	{ "Brazil/East",        "EST","EDT",  -3,  -3*60},
	{ "Atlantic/Stanley", 	"FKST","FKDT",-4,  -4*60},
	{ "America/Santiago", 	"CLST","CLDT",-4,  -4*60},
	{ "America/Atlantic", 	"AST", "ADT", -4,  -4*60},
	{ "Canada/Newfoundland","NST", "NDT", -3,  -(3*60+30)},
	{ "W. Greenland",       "WGST","WGDT",-3,  -3*60},
	{ "Greenwich Mean Time","GMT", "",     0,   0},
	{ "Europe/London",   	"GMT", "BST",  0,   0},
	{ "Europe/Dublin",   	"GMT", "IST",  0,   0},
	{ "Atlantic/Azores",    "AZOST","AZODT", -1, -1*60},
	{ "Western Europe",   	"WEST","WEDT", 0, 0},
	{ "Central Europe",   	"CEST","CEDT", 1, 1*60},
	{ "Europe/Moscow",   	"MSK","MSD",   3, 3*60},
	{ "Egypt",   	        "EEST","EEDT", 2, 2*60},
	{ "Asia/Tehran (appr.)","IRST","IRDT", 3, 3*60},
	{ "Asia/Baghdad",   	"AST","ADT",   3, 3*60},
	{ "Namibia", 	        "SAST", "SADT", 2, 2*60},
	{ "Asia/Vladivostok",   "VLAST","VLADT",10,10*60},
	{ "Asia/Magadan",   	"MAGST","MAGDT",11,11*60},
	{ "Asia/Kamchatka",   	"PETST","PETDT",12,12*60},
	{ "New Zealand",        "NZST", "NZDT",12,12*60},
	{ "Australia/Eastern",  "EST", "", 10,10*60},
	{ "Australia/Central",  "CST", "", 9,9*60+30},
	{ "Australia/Western",  "WST", "", 8,8*60},
	{ "Australia/S.Eastern","EST", "EDT", 10,10*60},
	{ "Australia/S.Central","CST", "CDT",9,9*60+30}
   	};

typedef struct {
  int hours;
  char zoneName[20];
} tz;
tz timeZones[] = {
                  {0*60,   "Z (GMT)"},
                  {1*60,   "A (GMT + 1:00)"},
                  {2*60,   "B (GMT + 2:00)"},
                  {3*60,   "C (GMT + 3:00)"},
                  {3.5*60, "C* (GMT + 3:30)"},
                  {4*60,   "D (GMT + 4:00)"},
                  {4.5*60, "D* (GMT + 4:30)"},
                  {5*60,   "E (GMT + 5:00)"},
                  {5.5*60, "E* (GMT + 5:30)"},
                  {6*60,   "F (GMT + 6:00)"},
                  {6.5*60, "F* (GMT + 6:30)"},
                  {7*60,   "G (GMT + 7:00)"},
                  {8*60,   "H (GMT + 8:00)"},
                  {9*60,   "I (GMT + 9:00)"},
                  {9.5*60, "I* (GMT + 9:30)"},
                  {10*60,  "K (GMT + 10:00)"},
                  {10.5*60,"K* (GMT + 10:30)"},
                  {11*60,  "L (GMT + 11:00)"},
                  {11.5*60,"L* (GMT + 11:30)"},
                  {12*60,  "M (GMT + 12:00)"},
                  {13*60,  "M* (GMT + 13:00)"},
                  {-1*60,  "N (GMT - 1:00)"},
                  {-2*60,  "O (GMT - 2:00)"},
                  {-3*60,  "P (GMT - 3:00)"},
                  {3.5*60, "P* (GMT - 3:30)"},
                  {-4*60,  "Q (GMT - 4:00)"},
                  {-5*60,  "R (GMT - 5:00)"},
                  {-6*60,  "S (GMT - 6:00)"},
                  {-7*60,  "T (GMT - 7:00)"},
                  {-8*60,  "U (GMT - 8:00)"},
                  {8.5*60, "U* (GMT - 8:30)"},
                  {-9*60,  "V (GMT - 9:00)"},
                  {9.5*60, "V* (GMT - 9:30)"},
                  {-10*60, "W (GMT - 10:00)"},
                  {-11*60, "X (GMT - 11:00)"},
                  {-12*60, "Y (GMT - 12:00)"}
                  };

typedef struct {
  char zoneName[20];
} tzNameType;
tzNameType tzNames[] = {
                  {"Z (GMT)"},
                  {"A (GMT + 1:00)"},
                  {"B (GMT + 2:00)"},
                  {"C (GMT + 3:00)"},
                  {"C* (GMT + 3:30)"},
                  {"D (GMT + 4:00)"},
                  {"D* (GMT + 4:30)"},
                  {"E (GMT + 5:00)"},
                  {"E* (GMT + 5:30)"},
                  {"F (GMT + 6:00)"},
                  {"F* (GMT + 6:30)"},
                  {"G (GMT + 7:00)"},
                  {"H (GMT + 8:00)"},
                  {"I (GMT + 9:00)"},
                  {"I* (GMT + 9:30)"},
                  {"K (GMT + 10:00)"},
                  {"K* (GMT + 10:30)"},
                  {"L (GMT + 11:00)"},
                  {"L* (GMT + 11:30)"},
                  {"M (GMT + 12:00)"},
                  {"M* (GMT + 13:00)"},
                  {"N (GMT - 1:00)"},
                  {"O (GMT - 2:00)"},
                  {"P (GMT - 3:00)"},
                  {"P* (GMT - 3:30)"},
                  {"Q (GMT - 4:00)"},
                  {"R (GMT - 5:00)"},
                  {"S (GMT - 6:00)"},
                  {"T (GMT - 7:00)"},
                  {"U (GMT - 8:00)"},
                  {"U* (GMT - 8:30)"},
                  {"V (GMT - 9:00)"},
                  {"V* (GMT - 9:30)"},
                  {"W (GMT - 10:00)"},
                  {"X (GMT - 11:00)"},
                  {"Y (GMT - 12:00)"}
                  };

//PROBLEM: gcc pads strings with nulls, SysFormPointerArrayToStrings inter
//prets each of these as 0-byte strings!  lots of blanks

static Boolean HandleNow(EventPtr event)
{
  DateTimeType oldDateTime;
  FormPtr form;

      form = FrmGetActiveForm();

      if (event->eType == nilEvent && newSunAngleInput.now)
      { 
        oldDateTime = newSunAngleInput.dateTime;
        TimSecondsToDateTime (TimGetSeconds(), &newSunAngleInput.dateTime);
        if (oldDateTime.minute != newSunAngleInput.dateTime.minute
            || oldDateTime.hour != newSunAngleInput.dateTime.hour)

            updateTime(form, true);

        if (oldDateTime.month != newSunAngleInput.dateTime.month
            || oldDateTime.day != newSunAngleInput.dateTime.day
            || oldDateTime.year != newSunAngleInput.dateTime.year)

            updateDate(form, true);
      }
     // we don't return true because we don't pretend to handle the
     // event.  this is because Astro.c doesn't... i think it might
     // be so everyone else sees the nil event.
     return false;
}

void updateDate(FormPtr form, Boolean update)
{
  Char *label;

  label = (Char *)CtlGetLabel((ControlPtr)GetObjectPtr(form, DatePushButton));
  DateToAscii (newSunAngleInput.dateTime.month,
   newSunAngleInput.dateTime.day,
   newSunAngleInput.dateTime.year,
   dfMDYWithSlashes,
   label);

//  FrmSetFocus(form, noFocus);
  if (update) 
   FrmDrawForm(form);
}
void updateTime(FormPtr form, Boolean update)
{
  Char *label;

  label = (Char *)CtlGetLabel((ControlPtr)GetObjectPtr(form, TimePushButton));
  TimeToAscii (newSunAngleInput.dateTime.hour,
   newSunAngleInput.dateTime.minute,
   tfColonAMPM,
   label);
//  FrmSetFocus(form, noFocus);
  if (update)
   FrmDrawForm(form);
}


Boolean SettingsFormHandleEvent(EventPtr event)
{
  Boolean handled = false;
  FormPtr form;
  ListPtr list;
  Int32 i;
  Char *t;

  char sunangleValues[90];

  Char **appNamesArray = NULL;
//  Char datetext[longDateStrLength];

  MemPtr ptrTable;

//  VoidPtr theListPtr;
//  Handle theListItems;
//  DWord theListCount;
//  VoidHand theRecHand;
//  char *theRecPtr;
  DateTimeType oldDateTime;

  form = FrmGetActiveForm();
  HandleNow(event);
  switch (event->eType)
  {
  case frmOpenEvent:
    SettingsFormInit(form);

    // Start with old (global) settings...
    newSunAngleInput.latitude = sunangleInput->latitude;
    newSunAngleInput.longitude = sunangleInput->longitude;
    newSunAngleInput.dateTime = sunangleInput->dateTime;
    newSunAngleInput.timeZone = sunangleInput->timeZone;
    newSunAngleInput.dst = sunangleInput->dst;
    newSunAngleInput.now = sunangleInput->now;
    newSunAngleInput.now = sunangleInput->now;

    
    // Initialize TimeZone list
    list = (ListPtr) GetObjectPtr(form, TZList);
    tmpH4 = DmGetResource('tSTL', TZStringTable);
    ptrTable = (MemPtr)MemHandleLock(tmpH4);
    ptrTable += StrLen(ptrTable)+1;
    ptrTable += sizeof(Word);
    //memStringList = SysFormPointerArrayToStrings(ptrTable+StrLen((Char *)ptrTable) + 3, 36);
    tmpH3 = SysFormPointerArrayToStrings(ptrTable, 36);
    appNamesArray = MemHandleLock(tmpH3);
    LstSetListChoices(list, appNamesArray, 36);
    LstSetHeight(list, 6);

    list = (ListPtr)GetObjectPtr(form, TZList);
    LstSetSelection (list, newSunAngleInput.timeZone);
    t = (Char *)CtlGetLabel ((ControlPtr)GetObjectPtr(form, TZTrigger));
    // why do i have to typecast this?  it returns const Char *.  why const? 
    StrCopy (t, LstGetSelectionText(list, LstGetSelection(list)));

    if (newSunAngleInput.latitude < 0) {
      newSunAngleInput.latitude = -newSunAngleInput.latitude;
      CtlSetValue (GetObjectPtr(form, LatNButton), 0);
      CtlSetValue (GetObjectPtr(form, LatSButton), 1);
    } else {
      CtlSetValue (GetObjectPtr(form, LatNButton), 1);
      CtlSetValue (GetObjectPtr(form, LatSButton), 0);
    }

    if (newSunAngleInput.longitude < 0) {
      newSunAngleInput.longitude = -newSunAngleInput.longitude;
      CtlSetValue (GetObjectPtr(form, LongEButton), 0);
      CtlSetValue (GetObjectPtr(form, LongWButton), 1);
    } else {
      CtlSetValue (GetObjectPtr(form, LongEButton), 1);
      CtlSetValue (GetObjectPtr(form, LongWButton), 0);
    }

    // Initialize Latitude and Longitude fields from record
    t = MemPtrNew(40);
    StrIToA(t, newSunAngleInput.latitude);
    SetFieldTextFromStr((FieldPtr)GetObjectPtr(form, LatField), t, true);
    StrIToA(t, newSunAngleInput.longitude);
    SetFieldTextFromStr((FieldPtr)GetObjectPtr(form, LongField), t, true);
    MemPtrFree(t);

    CtlSetValue(GetObjectPtr(form, DSTCheck), newSunAngleInput.dst);

    updateDate(form, false);
    updateTime(form, false);
    FrmDrawForm(form);
// LstSetSelection(lst, listSelection);
    // here's where you'd add a call to FrmSetFocus
    handled = true;
    break;

  case ctlSelectEvent:
  switch (event->data.ctlSelect.controlID) {
    case SettingsOKButton:
      if (! (t=FldGetTextPtr((FieldPtr)GetObjectPtr(form, LatField)))) {
        FrmAlert (LatAlert);
        break;
      } else {
        StrCopy(sunangleValues, t);
        i = StrAToI(sunangleValues);
        if (i<0 || i>90) {
          FrmAlert (LatAlert);
          break;
        }
      }
      newSunAngleInput.latitude = i;
      if (CtlGetValue(GetObjectPtr(form, LatSButton)))
        newSunAngleInput.latitude = -newSunAngleInput.latitude;

      if (! (t=FldGetTextPtr((FieldPtr)GetObjectPtr(form, LongField)))) {
        FrmAlert (LongAlert);
        break;
      } else {
        StrCopy(sunangleValues, t);
        i = StrAToI(sunangleValues);
        if (i<0 || i>180) {
          FrmAlert (LongAlert);
          break;
        }
      }
      newSunAngleInput.longitude = i; 
      if (CtlGetValue(GetObjectPtr(form, LongWButton)))
        newSunAngleInput.longitude = -newSunAngleInput.longitude;

      newSunAngleInput.timeZone = LstGetSelection((ListPtr)GetObjectPtr(form, TZList));

      if ( abs((newSunAngleInput.longitude/15) - ConvertTzone(newSunAngleInput.timeZone)) > 2) 
      {
        FrmAlert (LongTZAlert);
        break;
      }

      newSunAngleInput.dst = CtlGetValue(GetObjectPtr(form, DSTCheck));
      // All input has been validated... copy to global
      sunangleInput->latitude = newSunAngleInput.latitude;
      sunangleInput->longitude = newSunAngleInput.longitude;
      sunangleInput->dateTime = newSunAngleInput.dateTime;
      sunangleInput->timeZone = newSunAngleInput.timeZone;
      sunangleInput->dst = newSunAngleInput.dst;
      sunangleInput->now = newSunAngleInput.now;
      sunangleInput->now = newSunAngleInput.now;
      FrmGotoForm(ListingForm);
      handled = true;
      break;
    case SettingsCancelButton:
      FrmGotoForm(ListingForm);
      handled = true;
      break;
    case DatePushButton:
      oldDateTime = newSunAngleInput.dateTime;
      SelectDay(selectDayByDay, &newSunAngleInput.dateTime.month, &newSunAngleInput.dateTime.day, &newSunAngleInput.dateTime.year, "Select Day");
//      ErrDisplay (stringDay);
      updateDate(form, true);
      newSunAngleInput.now = false;
      break;
    case TimePushButton:
      oldDateTime = newSunAngleInput.dateTime;
      SelectOneTime (&newSunAngleInput.dateTime.hour, &newSunAngleInput.dateTime.minute, "Select Time");
      updateTime(form, true);
      newSunAngleInput.now = false;
      break;
    case NowButton:
      oldDateTime = newSunAngleInput.dateTime;
      TimSecondsToDateTime (TimGetSeconds(), &newSunAngleInput.dateTime);
      updateTime(form, true);
      updateDate(form, true);
      newSunAngleInput.now = true;
      break;
    case b_Info:
      FrmGotoForm(InfoForm);
      handled = true;
      break;
    case b_Alt:
      FrmGotoForm(AltForm);
      handled = true;
      break;
    case b_Compass:
      FrmGotoForm(CompassForm);
      handled = true;
      break;
    case b_Listing:
      FrmGotoForm(ListingForm);
      handled = true;
      break;
    case b_Settings:
      handled = true;
      break;
    }
    break;
  case ctlEnterEvent:
    break;
  case frmCloseEvent:
    if(tmpH3) {
      MemHandleUnlock(tmpH3);
      MemHandleFree(tmpH3);  // why do i need to do this?
    }
    if(tmpH4) {
      MemHandleUnlock(tmpH4);
      DmReleaseResource(tmpH4);  // why do i need to do this?
    }
    SettingsFormDeinit(FrmGetActiveForm());
    handled = false;
    break;

  default:
    break;
  }
  return handled;
}
