/*
 Copyright (c) 2000-2001, Neil Rhodes and Julie McKeehan
    neil@pobox.com
 All rights reserved.

From the book "Palm Programming, the Developer's Guide, 2nd edition"
by O'Reilly.

Permission granted to use this file however you see fit.
*/

#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "Utils.h"
#include "ListingForm.h"
#include "SunAngle.h"
#include "StringMgr.h"
#include "MathLib.h"


double cos_table[1000];
double sin_table[1000];
double asin_table[1001];
double incr;
double aincr;
UInt16 j;

void initTrigTables()
{
  double i;
  return;
  incr = 2.0*PI/500.0;
  aincr = 1.0/250.0;
  j=0;
  for (i=0.0;j<500;i=i+incr)
  {
    cos_table[j] = cos(i);
    sin_table[j] = sin(i);
    j++;
  }
  j=0;
//  for (i=-1;i<=1;i=(double)i+(double)aincr) // we need -1.0..1.0 inclusive
  for (j=0;j<=500;j++) // we need -1.0..1.0 inclusive
  {
    //asin_table[j++] = asin(i);
    asin_table[j] = asin((j-250.0)*aincr);
  }
}

double tcos(double theta)
{
  return cos(theta);
  while (theta < 0) 
    theta = 2.0*PI + theta;
  while (theta > 2.0*PI) 
    theta = theta - 2.0*PI;
  if (theta <= PI/2) 
    return cos_table[(int)((double)(theta)/(double)incr)];
  if (theta <= PI)
    return cos_table[(int)((double)(PI-theta)/(double)incr)];
  if (theta <= 3*PI/2)
    return -cos_table[(int)((double)(theta-PI)/(double)incr)];
  return -cos_table[(int)((double)(2*PI-theta)/(double)incr)];
}
double tsin(double theta)
{
  return sin(theta);
  if (theta < 0) 
    theta = 2*PI + theta;
  while (theta > 2.0*PI) 
    theta = theta - 2.0*PI;
  return sin_table[(int)(theta/incr)];
}

double tasin(double x)
{
  return asin(x);
  return asin_table[(int)((x+1.0)/aincr)];
}

Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
  UInt32 romVersion;

  // See if we're on in minimum required version of the ROM or later.
  FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
  if (romVersion < requiredVersion) {
    UInt16 safeToCallAlertFlags;
    
    safeToCallAlertFlags = 
      sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp;
    if ((launchFlags & (safeToCallAlertFlags)) == safeToCallAlertFlags) {
      FrmAlert (RomIncompatibleAlert);
    
      // Pilot 1.0 will continuously relaunch this app unless we switch to 
      // another safe one.
      if (romVersion < sysMakeROMVersion(2,0,0,sysROMStageRelease,0))
        AppLaunchWithCommand(sysFileCDefaultApp, 
          sysAppLaunchCmdNormalLaunch, NULL);
    }
    
    return (sysErrRomIncompatible);
  }

  return errNone;
}


void *GetObjectPtr(FormPtr form, UInt16 objectID)
{
  return FrmGetObjectPtr(form, FrmGetObjectIndex(form, objectID));
}

void *GetObjectPtrFromActiveForm(UInt16 objectID)
{
  return GetObjectPtr(FrmGetActiveForm(), objectID);
}


Err SetFieldTextFromStr(FieldPtr field, const Char *s, Boolean redraw)
{
   MemHandle      h;
   
   h = FldGetTextHandle(field);
   if (h) {
     Err  err;
     
     FldSetTextHandle(field, NULL);
     err = MemHandleResize(h, StrLen(s) + 1);
     if (err != errNone) {
       FldSetTextHandle(field, h);  // restore handle
       return err;
     }
   } else {
// g.b. how does this get freed?
     h = MemHandleNew(StrLen(s) + 1);
     if (!h)
       return memErrNotEnoughSpace;
   }
   // at this point, we have a handle of the correct size

   // copy the string to the locked handle.
   StrCopy((Char *) MemHandleLock(h), s);
   // unlock the string handle.
   MemHandleUnlock(h);
   
   FldSetTextHandle(field, h);
   if (redraw)
     FldDrawField(field);
   return errNone;
} 

void SetFieldTextFromHandle(FieldPtr field, MemHandle newHandle, Boolean redraw){
  MemHandle  oldHandle;
  oldHandle = FldGetTextHandle(field);
  FldSetTextHandle(field, newHandle);
  if (redraw)
    FldDrawField(field);
  if (oldHandle)
  MemHandleFree(oldHandle);
}
 
void DisplayStringsInAlert(const Char *s1, const Char *s2)
{
  FrmCustomAlert(InfoAlert, s1, s2, NULL);
}

double ConvertTzone (int tZone)
{
  double tzones[36] = {0,1,2,3,3.5,4,4.5,5,5.5,6,6.5,7,8,9,9.5,10,10.5,11,11.5,12,13,-1,-2,-3,-3.5,-4,-5,-6,-7,-8,-8.5,-9,-9.5,-10,-11,-12};
  return tzones[tZone]; 
}
void CalcSunangle (sunangleInputPtr ins, 
                   sunangleOutputType* now, 
                   Boolean fullCalc) 
{

  int i;
  double latitude = (double)ins->latitude; 
  double longitude = (double)ins->longitude;
  double month = (double)ins->dateTime.month;
  double date = (double)ins->dateTime.day;
  int year = ins->dateTime.year;
  double hour = (double)ins->dateTime.hour;
  double minute = (double)ins->dateTime.minute;
  double tz = ConvertTzone(ins->timeZone);
  int ds = ins->dst;
  double days;
  double b;
  double eot;
  double localTime;
  double localSolarTime;
  double declination;        // output
  double altitude;           // output
  double secondsBeforeNoon;
  double minutesBeforeNoon;
  double hourAngle;
  double azimuthAngle;
  double compassBearing;     // output
  double sunRiseHourAngle;   // output
  double sunRiseMinutes;
  double sunRiseLsoT;
  double sunSetLsoT;
  double sunrise;            // output
  double sunset;        // output

  double degPerRad = 360/(2*3.141592654);
  double radPerDeg = (2*3.141592654)/360;

//  MemPtr inputsString; //[100];
//  MemPtr str; //[10];
 
  int totalDays[12] = {31,28,31,30,31,30,31,31,30,31,30,31};
  
//  inputsString = MemPtrNew (1000);
//  str = MemPtrNew (100);
/*
  StrCopy(inputsString, "lat:");
  printDouble (latitude, str);
  StrCat(inputsString, str); 
  StrCat(inputsString, "long:");
  printDouble (longitude, str);
  StrCat (inputsString, str);
  StrCat(inputsString, "month:");
  printDouble (month, str);
  inputsString = StrCat (inputsString, str);
*/
//  printDouble (tz, str);
/*
  StrPrintF(inputsString, "Lat: %ld\nLong: %ld\nDate: %d/%d/%d\nTZ: %s\nDST: %d\nNow: %d", 
   ins->latitude, 
   ins->longitude, 
   ins->dateTime.month, 
   ins->dateTime.day, 
   ins->dateTime.year, 
   str,
   ins->dst, 
   ins->now); 
  MemPtrFree(inputsString);
  MemPtrFree(str);
  DisplayStringsInAlert("SunAngle Inputs:\n", inputsString);
*/
  // Could replace this with DateToDays - DateToDays(1/1/1904)
  days = 0;
  for (i=0;i<month-1;i++) {
    days += totalDays[i];
  }
  days += date;
  if (!(year % 4))
    days++;  // leap year, forgetting about centuries, etc.

  b = 360*(days-81)/365;
  eot = 9.87*sin(2*b*radPerDeg)-7.53*cos(b*radPerDeg)-1.5*sin(b*radPerDeg);

  // *** missing some logic from the excel spreadsheet in here...
  localTime = 3600*hour+60*minute-3600.0*ds;
  localSolarTime = localTime+(4.0*60.0*(longitude-(tz*15.0)))+eot*60.0;
  declination = 23.45*sin(radPerDeg*(360.0/365.0)*(284.0+days));
  if (latitude < 0)
    declination = -1*declination;

  secondsBeforeNoon = (12.0*3600.0)-localSolarTime;
  minutesBeforeNoon = secondsBeforeNoon/60;
  hourAngle = minutesBeforeNoon/4;
  altitude = asin((cos(radPerDeg*latitude)*cos(radPerDeg*declination)
                    *cos(radPerDeg*hourAngle))
                   +(sin(radPerDeg*latitude)*sin(radPerDeg*declination)))
             *degPerRad;
  if (fullCalc) {
  azimuthAngle = acos((sin(radPerDeg*altitude)*sin(radPerDeg*latitude)
                        -sin(radPerDeg*declination))
                       /(cos(radPerDeg*altitude)*cos(radPerDeg*latitude)))
                 *degPerRad;
  if (hourAngle < 0)
    azimuthAngle = -1*azimuthAngle;
  compassBearing = 180-azimuthAngle;
  sunRiseHourAngle = acos(-1*(sin(radPerDeg*latitude)*sin(radPerDeg*declination)
                               -sin(-0.8333*radPerDeg))/
                             (cos(radPerDeg*latitude)
                              *cos(radPerDeg*declination)) )
                     *degPerRad;

  sunRiseMinutes = 4*sunRiseHourAngle;
  sunRiseLsoT = (12*60.0-sunRiseMinutes);
  sunSetLsoT = (12*60.0+sunRiseMinutes);

  sunrise = sunRiseLsoT - (4*(longitude-(tz*15))+eot)+ds*60;
  sunset = sunSetLsoT - (4*(longitude-(tz*15))+eot)+ds*60;
  (*now).declination = declination;
  (*now).azimuth = azimuthAngle;
  (*now).sunRise.hours = (int)sunrise/60;
  (*now).sunRise.minutes = sunrise - ((int)sunrise/60)*60;  // use mod
  (*now).sunSet.hours = (int)sunset/60;
  (*now).sunSet.minutes = sunset - ((int)sunset/60)*60;  // use mod
  (*now).bearing = 180-azimuthAngle;
  }
  (*now).altitude = altitude;
 
  return;
}

Int16 double2Int16(double x) 
{
  return (Int16)(x + (x>0?0.5:-0.5));
}
